import requests
from io import BytesIO
import os
import mimetypes
import re

MIME_REGEX = re.compile(r'^[a-zA-Z0-9!#$&^_.+-]+/[a-zA-Z0-9!#$&^_.+-]+$')

def validate_content_type(content_type=None, file_name=None):
    # If user passed content_type, validate it
    if content_type:
        if not MIME_REGEX.match(content_type):
            raise ValueError(f"Invalid content_type format: {content_type}")
        
        # Known MIME types list from Python
        known_mime_types = set(mimetypes.types_map.values())
        if content_type in known_mime_types:
            return content_type
        else:
            # Allow custom but valid format → fallback
            return content_type  # or raise error if you want strict mode
    
    # If not provided, try guessing from file name
    if file_name:
        guessed, _ = mimetypes.guess_type(file_name)
        return guessed or "application/octet-stream"
    
    # Fallback
    return "application/octet-stream"

def upload_to_s3(token, tapestry_id, blob, app_name, doc_name,  content_type=None ):
    url = "https://inthepicture.org/admin/upload_to_s3"
    headers = {
        "accept": "application/json, text/plain, */*",
        "authorization": f"Bearer {token}",
    }

    # ✅ Check if blob is a Jupyter FileUpload-like dict with MIME type
    if isinstance(blob, dict) and "content" in blob:
        file_data = blob["content"]
        if isinstance(file_data, str):
            file_data = file_data.encode('utf-8')
        # Use blob's own type if available and no explicit content_type
        if content_type is None and "type" in blob and blob["type"]:
            content_type = blob["type"]
    else:
        # Handle normal file path or raw content
        if isinstance(blob, str) and os.path.isfile(blob):
            file_data = open(blob, "rb")
        else:
            if isinstance(blob, str):
                blob = blob.encode("utf-8")
            file_data = BytesIO(blob)

    # ✅ Validate or infer content type (from explicit param or filename)
    content_type = validate_content_type(content_type, file_name=doc_name)

    files = {
        "blob": (doc_name, file_data, content_type),
    }
    data = {
        "tapestry_id": tapestry_id,
        "app_name": app_name,
        "doc_name": doc_name,
    }
    response = requests.post(url, headers=headers, files=files, data=data)

    # Close file if it was opened from disk
    if isinstance(blob, str) and os.path.isfile(blob):
        file_data.close()

    try:
        resp_json = response.json()
    except Exception:
        resp_json = {}

    if response.status_code == 200 and resp_json.get("success", False):
        return resp_json
    else:
        message = resp_json.get("message", response.text)
        return {
            "success": False,
            "code": response.status_code,
            "message": message,
            "body": {}
        }



import requests

def generate_report(tapestry_id, group_id, document_ids, is_chat=False, user_prompt=""):
    url = "https://inthepicture.org/admin/report_generation"
    payload = {
        "tapestry_id": tapestry_id,
        "group_id": group_id,
        "document_ids": document_ids,
        "is_chat": is_chat,
        "user_prompt": user_prompt
    }

    response = requests.post(url, json=payload)
    try:
        resp_json = response.json()
    except Exception:
        resp_json = {}

    if response.status_code == 200 and resp_json.get("success", False):
        return resp_json
    else:
        message = resp_json.get("message", response.text)
        return {
            "success": False,
            "code": response.status_code,
            "message": message,
            "body": {}
        }



import json
from .fetch_folder_data import fetch_folder_data
from .image_to_text import image_to_text
from .fetch_group_data import fetch_group_data
from .selected_document_data import selected_document_data
from .fetch_ai_chat import fetch_ai_chat
from .fetch_sticky_notes import fetch_sticky_notes
from .fetch_documents import fetch_documents
from .ask_ai_question import ask_ai_question
from .search_group import search_group
from .change_tapestry_details import change_tapestry_details
from .set_load_Status import set_load_Status
from .upload_file import upload_file
from .delete_s3_doc import delete_s3_doc
from .fetch_group_document import fetch_group_document
from .fetch_group_list import fetch_group_list
from .list_s3_doc import list_s3_doc
from .update_s3_doc import update_s3_doc
from .generate_report import generate_report
from .upload_to_s3 import upload_to_s3

def hello():
    print("Hello from Tapestry!")


__all__ = [
    "fetch_folder_data",
    "image_to_text",
    "fetch_group_data",
    "selected_document_data",
    "fetch_ai_chat",
    "fetch_sticky_notes",
    "fetch_documents",
    "ask_ai_question",
    "search_group",
    "change_tapestry_details",
    "set_load_Status",
    "upload_file",
    "delete_s3_doc",
    "fetch_group_document",
    "fetch_group_list",
    "list_s3_doc",
    "update_s3_doc",
    "generate_report",
    "upload_to_s3",
]




import requests

def ask_ai_question(token, question, tapestry_id, session_id=None, group_ids=None, document_name=None,ai_type=None):
    url = "https://inthepicture.org/admin/ask_ai_question"
    headers = {
        "accept": "application/json, text/plain, */*",
        "authorization": f"Bearer {token}",
        "content-type": "application/json",
    }
    
    data = {
        "question": question,
        "tapestry_id": tapestry_id
    }

    # Optional fields
    if session_id is not None:
        data["session_id"] = session_id
    if group_ids:
        data["group_ids"] = group_ids
    if document_name:
        data["documentName"] = document_name
    if ai_type:
        data["type"] = ai_type

    response = requests.post(url, headers=headers, json=data)

    try:
        resp_json = response.json()
    except Exception:
        resp_json = {}

    if response.status_code == 200 and resp_json.get("success", False):
        return resp_json
    else:
        message = resp_json.get("message", response.text)
        return {
            "success": False,
            "code": response.status_code,
            "message": message,
            "body": {}
        }


import requests

def fetch_group_list(tapestry_id):
    url = "https://inthepicture.org/admin/fetch_group_list"
    params = {
        "tapestry_id": tapestry_id
    }

    response = requests.get(url, params=params)
    try:
        resp_json = response.json()
    except Exception:
        resp_json = {}

    if response.status_code == 200 and resp_json.get("success", False):
        return resp_json
    else:
        message = resp_json.get("message", response.text)
        return {
            "success": False,
            "code": response.status_code,
            "message": message,
            "body": {}
        }



import requests
from io import BytesIO
import os

def update_s3_doc(token, doc_id, blob, content_type=None):
    url = "https://inthepicture.org/admin/update_s3_doc"
    headers = {
        "accept": "application/json, text/plain, */*",
        "authorization": f"Bearer {token}",
    }

    # Determine if blob is a file path or raw content
    if isinstance(blob, str) and os.path.isfile(blob):
        file_like = open(blob, "rb")
    else:
        if isinstance(blob, str):
            blob = blob.encode('utf-8')
        file_like = BytesIO(blob)

    files = {
        "blob": ("file", file_like, content_type),
    }
    data = {
        "doc_id": doc_id,
    }

    response = requests.post(url, headers=headers, files=files, data=data)

    # Close file if opened from disk
    if isinstance(blob, str) and os.path.isfile(blob):
        file_like.close()

    try:
        resp_json = response.json()
    except Exception:
        resp_json = {}

    if response.status_code == 200 and resp_json.get("success", False):
        return resp_json
    else:
        return {
            "success": False,
            "code": response.status_code,
            "message": resp_json.get("message", response.text),
            "body": {}
        }



import requests

def set_load_Status(token, session_id):
    url =  "https://inthepicture.org/admin/set_load_Status"
    headers = {
        "accept": "application/json, text/plain, */*",
        "authorization": f"Bearer {token}",
        "content-type": "application/json",
    }
    data = {
        "session_id": session_id,
    }
    
    response = requests.post(url, headers=headers, json=data)
    try:
        resp_json = response.json()
    except Exception:
        resp_json = {}

    if response.status_code == 200 and resp_json.get("success", False):
        return resp_json
    else:
        message = resp_json.get("message", response.text)
        return {
            "success": False,
            "code": response.status_code,
            "message": message,
            "body": {}
        }
 


import requests

def fetch_folder_data(token, folder_details):
    folder_name = folder_details.get("name")
    tapestry_id = folder_details.get("tapestry_id")

    url =  "https://inthepicture.org/admin/library"
    headers = {
        "accept": "application/json, text/plain, */*",
        "authorization": f"Bearer {token}",
        "content-type": "application/json",
    }
    data = {
        "page": 1,
        "active": "grid",
        "group_id": [],
        "tapestry_id": tapestry_id,
        "parent": folder_name,
    }
    
    response = requests.post(url, headers=headers, json=data)
    try:
        resp_json = response.json()
    except Exception:
        resp_json = {}

    if response.status_code == 200 and resp_json.get("success", False):
        return resp_json
    else:
        message = resp_json.get("message", response.text)
        return {
            "success": False,
            "code": response.status_code,
            "message": message,
            "body": {}
        }
token="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6Nywicm9sZSI6bnVsbCwiYWRtaW5faWQiOjAsIm90cCI6MCwiaXNfdmVyaWZpZWQiOjAsIm9yZ19pZCI6MCwiaXNfYmxvY2siOjAsInBhc3N3b3JkIjoiIiwibmFtZSI6Ik1laHVsIEthdXNoYWwiLCJmaXJzdF9uYW1lIjoiTWVodWwiLCJzdHJpcGVfY3VzdG9tZXJfaWQiOiJjdXNfU255Z3lobzJUd1dZUlEiLCJpc193ZWxjb21lX21lc3NhZ2Vfc2hvd24iOjEsImlzX3B1YmxpYyI6MCwiYWJvdXQiOm51bGwsInZvaWNlIjoiSm9hbm5hIiwiaW1hZ2UiOiIiLCJsYXN0X25hbWUiOiJLYXVzaGFsIiwibmlja19uYW1lIjoiIiwiZW1haWwiOiJtZWh1bC5kdWNrdGFsZUBnbWFpbC5jb20iLCJmb3Jnb3RQYXNzd29yZEhhc2giOiIiLCJjaGF0X2lkIjoiIiwiZ29vZ2xlX2lkIjoiMTEwMzYxMDI2MjYwODIxNjQxMzcxIiwic2VjcmV0X2tleSI6InFuakg1emNqd2ZQVDRYMVZialpFbExmVCIsInBob25lIjoiIiwidGltZVpvbmUiOiJVVEMiLCJtZmFfdHlwZSI6MywiZ29vZ2xlX3NlY3JldCI6IiIsImlzX3FyX3NjYW5uZWQiOjAsImNyZWF0ZWRBdCI6IjIwMjUtMDgtMDRUMTE6Mzk6MTIuMDAwWiIsInVwZGF0ZWRBdCI6IjIwMjUtMDgtMjVUMTI6MTc6MTYuMDAwWiIsImlzX21vZGVyYXRvciI6MCwidW5pcXVlTmFtZSI6bnVsbCwiaWF0IjoxNzU2MTI0MjM2LCJleHAiOjE3NTYyMTA2MzZ9.zu4mRF_8kNuQp4Mc637PsTpLMaTWhIC6ZREnQemgVzA"
folder_details={"tapestry_id":13,"name":"fun_activity"}
res=fetch_folder_data(token,folder_details)
print(res)